# include <xc.h>
# include "uart.h"

#define UARTS_CLK_RATE              PBCLK_SET_100MHZ
#define UARTS_CLK_RATE              PBCLK_SET_100MHZ

// Defines

void Delay10uS(int num)
{
    T2CON = 0x8000; //enable TMR1 with 1:1   prescaler, clock used is FCY = 4000000

    while (num > 0) //wait for specified number of multiples of 100 microseconds
    {
        TMR2 = 0;

        while( TMR2 < (10000000/10000)); //wait 10 microseconds

        num = num-1;
    }
}

/* UART1Configure() sets up the UART2 for the most standard and minimal operation
 * Enable TX and RX lines, 8 data bits, no parity, 1 stop bit, idle when HIGH
 
 * Input: Desired Baud Rate
 * Output: Actual Baud Rate from baud control register U2BRG after assignment*/

int UART1Configure( int desired_baud )
{
    U1MODE = 0x8000; // Enable UART for 8-bit data
// No Parity, 1 Stop bit
U1STASET = 0x1400; // Enable Transmit and Receive

    U1MODE = 0;                      // disable autobaud, TX and RX enabled only, 8N1, idle=HIGH
    U1STA  = 0x1400;                 // enable TX and RX
    U1BRG  = Baud2BRG(desired_baud); // U1BRG = (FPb / (16*baud)) - 1

    // Calculate actual assigned baud rate

    int actual_baud = SYSCLK / (16 * (U1BRG+1));
 
    return actual_baud;
    
} // END UART1Configure()

/* SerialTransmit() transmits a string to the UART2 TX pin MSB first
 *
 * Inputs: *buffer = string to transmit */

void SerialTransmit( char * str )
{
    while( *str!='\0' )
    {
        while( U1STAbits.UTXBF );   // wait while TX buffer full

        U1TXREG = *str;             // send single character to transmit buffer

        str++;
    }

    while( !U1STAbits.TRMT);        // wait for last transmission to finish
}

unsigned int SerialReceive(char *buffer, unsigned int max_size)
{
    unsigned int num_char = 0;
 
    /* Wait for and store incoming data until either a carriage return is received
     *   or the number of received characters (num_chars) exceeds max_size */
    while(num_char < max_size)
    {
        while( !U1STAbits.URXDA);   // wait until data available in RX buffer
        *buffer = U1RXREG;          // empty contents of RX buffer into *buffer pointer
 
        // insert nul character to indicate end of string
        if( *buffer == '\r'){
            *buffer = '\0';     
            break;
        }
 
        buffer++;
        num_char++;
    }
 
    return num_char;
} // END SerialReceive()

int uart_read(uint8_t * data, size_t len) {
  int index = 0; // note: the provided len will never be greater than MAX_INT, so we can
                 // safely cast len to an int
  while(U3STAbits.URXDA && index != (int)len) {
    data[index] = U3RXREG;
    ++index;
  }
  return index == 0 ? -1 : index;
}

void ReadUART1(char * message, int maxLength) 
{
  char data = 0;
  
  int complete = 0, num_bytes = 0;
  
  // loop until you get a '\r' or '\n'
  
  while (!complete) 
  {
    if (U1STAbits.URXDA) { // if data is available
      data = U1RXREG;      // read the data
      if ((data == '\n') || (data == '\r')) {
        complete = 1;
      } else {
        message[num_bytes] = data;
        ++num_bytes;
        // roll over if the array is too small
        if (num_bytes >= maxLength) {
          num_bytes = 0;
        }
      }
    }
  }
  // end the string
  message[num_bytes] = '\0';
}

void WriteUART1(const char * string) {
  while (*string != '\0') {
      
    while (U1STAbits.UTXBF) {
      ; // wait until tx buffer isn't full
    }
    
    U1TXREG = *string;
    
    ++string;
    
  }
}

char getc(void)
{
	int temp = 0;
    
	if (U1STAbits.OERR == 1)
		U1STAbits.OERR = 0;
    
	while (U1STAbits.URXDA == 0);

	if (U1STAbits.FERR == 1 || U1STAbits.PERR == 1)
	{
		temp = U1RXREG;
        
		return 0;
	}

	return U1RXREG;
}

int getc2( char * uart_buf  )
{
    int index = 0;
    
    while (U1STAbits.URXDA == 1 && index < 10 ) 
    {
        char byte = getc();
        
        if( byte )
        {
            uart_buf[index] = getc();
        
            index ++;
        }
    }

    return index;
}

void writeChar(char data)
{
	while (U1STAbits.UTXBF == 1);
	U1TXREG = data;
}

void FlushUartReceiver()
{
    while(U1STAbits.URXDA) 
    {
        char temp = U1RXREG;   // Flush the RX Buffer
    }
    
    return;
}

void ReceiveUART1(char * ptrByte)
{
    while(1)
    {
        /* check for receive errors */
        if(U1STAbits.FERR == 1) { continue; }

        /* must clear the overrun error to keep uart receiving */
        if(U1STAbits.OERR == 1) { U1STAbits.OERR = 0; continue; }

        /* get the data */
        if(U1STAbits.URXDA == 1)
        {
            * ptrByte = U1RXREG;
            
            break;
        }
    }
}
